package com.persagy.account.manage;

import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.persagy.account.pojo.vo.account.SaasAccountDeleteVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.exception.BusinessException;
import com.persagy.common.model.BaseGroupModel;
import com.persagy.common.utils.StringUtil;
import com.persagy.account.mq.CommonTopicProducer;
import com.persagy.account.pojo.dto.SaasAccount;
import com.persagy.account.pojo.vo.auth.SaasAccountRoleVO;
import com.persagy.account.service.ISaasAccountProjectService;
import com.persagy.account.service.ISaasAccountRoleService;
import com.persagy.account.service.ISaasAccountService;
import com.persagy.security.util.BouncycastleCipher;

import cn.hutool.core.collection.CollectionUtil;

/**
 * @version 
 * @description 
 * @company persagy
 * @author zhangqiankun
 * @since 2021年3月16日:	上午10:31:06
 */
@Component
public class SaasAccountHandler {

    @Autowired
    private BouncycastleCipher bouncycastleCipher;
    
    @Autowired
    private ISaasAccountService saasAccountService;
    
	@Autowired
	private CommonTopicProducer commonTopicProducer;
    
	@Autowired
    private ISaasAccountRoleService saasAccountRoleService;
	
    @Autowired
    private ISaasAccountProjectService saasAccountProjectService;
	
	/**
	 * 账号新增，新增业务集团超管账号的时候，发送mq消息，且关联默认角色
	 * 
	 * @param saasAccount
	 * @param casType
	 * @param roles
	 * @return
	 */
	@Transactional
	public boolean createSaasAccount(SaasAccount saasAccount, String casType, List<SaasAccountRoleVO> roles) {
		boolean result = this.saasAccountService.save(saasAccount);
		// 新增成功，且是业务系统账号的超管，同步更新权限数据
		if (result && SaasCommonConstant.STR_STATUS_1.equals(saasAccount.getAccountBelong())
				&& SaasCommonConstant.STR_STATUS_1.equals(saasAccount.getAccountType())
				&& StringUtil.isNotBlank(saasAccount.getGroupCode())) {
			this.addAccountCasUpdateData(saasAccount.getId(), saasAccount.getGroupCode(), saasAccount);
		}
		// 判断是否需要添加账号-角色关联关系
		if (result && CollectionUtil.isNotEmpty(roles)) {
			this.saasAccountRoleService.resetSaasAccountRole(saasAccount.getGroupCode(), saasAccount.getId(), casType, roles);
		}
		return result;
	}
	
	/**
	 * 更新账号信息
	 * 
	 * @param saasAccount
	 * @param casType
	 * @param roles
	 * @return
	 */
	@Transactional
	public boolean updateSaasAccount(SaasAccount saasAccount, String casType, List<SaasAccountRoleVO> roles) {
		LambdaUpdateWrapper<SaasAccount> updateWrapper = new SaasAccount.Builder().createUpdateWrapper()
				.idEq(saasAccount.getId()).groupCodeEq(saasAccount.getGroupCode()).builderUpdateWrapper();
		boolean exists = this.saasAccountService.update(saasAccount, updateWrapper);
		if (exists) {
			// 如果更新成功，且是停用的时候，移除token
			if (SaasCommonConstant.STATUS_0 == saasAccount.getValid()) {
				this.bouncycastleCipher.remove(saasAccount.getId());
			}
			// 只有集团编码不为空时，才会发送mq消息
			if (StringUtil.isNotBlank(saasAccount.getGroupCode())) {
				this.commonTopicProducer.sendAccountInfo(saasAccount, SaasCommonConstant.UPDATE_TYPE);
			}
			// 判断是否需要添加账号-角色关联关系
			if (CollectionUtil.isNotEmpty(roles)) {
				this.saasAccountRoleService.resetSaasAccountRole(saasAccount.getGroupCode(), saasAccount.getId(), casType, roles);
			}
		}
		return exists;
	}
	
    /**
     * 删除账号信息，账号有恢复操作，这里不级联删除
     * 
     * @param deleteModel
     * @return
     */
	@Transactional
	public boolean deleteSaasAccount(BaseGroupModel deleteModel) {
		// 逻辑删除
    	LambdaUpdateWrapper<SaasAccount> updateWrapper = new SaasAccount.Builder().createUpdateWrapper()
				.idEq(deleteModel.getAccountId()).groupCodeEq(deleteModel.getGroupCode())
				.validEq(SaasCommonConstant.STATUS_1).builderUpdateWrapper();
    	updateWrapper.set(SaasAccount::getUpdateUser, deleteModel.getAccountId());
    	updateWrapper.set(SaasAccount::getValid, SaasCommonConstant.STATUS_0);
    	boolean result = this.saasAccountService.update(updateWrapper);
    	/*if (result) {
    		// 删除本方账号角色菜单关联数据
    		LambdaQueryWrapper<SaasAccountRole> queryWrapper = new SaasAccountRole.Builder().createQueryWrapper()
    				.accountIdEq(deleteModel.getAccountId()).groupCodeEq(deleteModel.getGroupCode()).builderQueryWrapper();
    		this.saasAccountRoleService.remove(queryWrapper);
    	}*/
    	return result;
	}

	/**
	 * 集团关联业务超管账号
	 * 
	 * @param model
	 * @return
	 */
	@Transactional
	public boolean addSaasAccountGroup(BaseGroupModel model) {
		LambdaUpdateWrapper<SaasAccount> updateWrapper = new SaasAccount.Builder().createUpdateWrapper().idEq(model.getAccountId())
				.accountBelongEq(SaasCommonConstant.STR_STATUS_1).accountTypeEq(SaasCommonConstant.STR_STATUS_1).builderUpdateWrapper();
    	updateWrapper.set(SaasAccount::getUpdateUser, model.getAccountId());
    	updateWrapper.set(SaasAccount::getGroupCode, model.getGroupCode());
    	boolean result = this.saasAccountService.update(updateWrapper);
    	if (result) {
    		this.addAccountCasUpdateData(model.getAccountId(), model.getGroupCode(), null);
		}
		return result;
	}
	
	/**
	 * 添加账号与集团的关联关系时，需要同步更新的数据
	 * @param accountId
	 * @param groupCode
	 */
	private void addAccountCasUpdateData(String accountId, String groupCode, SaasAccount saasAccount) {
		// 插入，默认的账号与业务超管角色关联关系
		boolean result = this.saasAccountRoleService.addDeafultAccountRole(accountId, groupCode);
		if (!result) {
			throw new BusinessException("账号与默认角色关联失败");
		}
		// 插入账号、集团、区域信息
		result = this.saasAccountProjectService.addDefaultAccountArea(accountId, groupCode);
		if (!result) {
			throw new BusinessException("账号默认的未分区权限添加失败");
		}
		// 发送mq消息，且成功关联了，才发送消息
		if (saasAccount == null) {
			saasAccount = this.saasAccountService.getOne(accountId, groupCode, null);
		}
		this.commonTopicProducer.sendAccountInfo(saasAccount, SaasCommonConstant.CREATE_TYPE);
	}


	/**
	 * @Description: TODO
	 * @Author: xgq
	 * @Date: 2021/4/29 21:46
	 * @param deleteModel:
	 * @return: boolean
	 **/
	@Transactional
    public boolean deleteSaasAccountListByIds(SaasAccountDeleteVO deleteModel) {
        LambdaQueryWrapper<SaasAccount> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SaasAccount::getAccountBelong, deleteModel.getAccountBelong());
        queryWrapper.eq(SaasAccount::getAccountType, deleteModel.getAccountType());
        queryWrapper.eq(SaasAccount::getAccountSource, deleteModel.getAccountSource());
        queryWrapper.eq(SaasAccount::getGroupCode,deleteModel.getGroupCode());
        queryWrapper.in(SaasAccount::getId,deleteModel.getIds());
        boolean remove = saasAccountService.remove(queryWrapper);
        return remove;
    }
}
